
#include <BLE2902.h>
#include <BLEDevice.h>
#include <BLEServer.h>
#include <BLEUtils.h>
//// above used only for BLE
#include <driver/i2s.h>
#include <Arduino.h>
#include "M5StickCPlus.h"
#include "esp_pm.h"
#include "RunningAverage.h"

RunningAverage myRA(64);       //was 10   35 better
RunningAverage myRAtheta(64);  //was 10   35 better
int samples = 0;
int samplestheta = 0;
volatile float swing = 4;     //calibrate roll gain : 4  OK
float pointer = (swing * 3);  //calibrate inclination pointer gain  (swing * 3)   OK
int span = 8;                 //span of anti-cant travel incl mid-point  (Ex: -4 and +4 from setpoint)
float offset = 0;             //compensate for tilt in mount for sensor (within limits: not moret than 15(?) degrees)
volatile float temp;          //hold value so will not change later

TFT_eSprite Disbuff = TFT_eSprite(&M5.Lcd);

void setup() {
  M5.begin();
  M5.Imu.Init();
  Wire.begin(32, 33);
  myRA.clear();  // explicitly start clean  AVG
  myRAtheta.clear();
  M5.Lcd.setRotation(1);
  Disbuff.setTextSize(1);
  Disbuff.setTextColor(TFT_WHITE);
  Disbuff.createSprite(240, 135);
  Disbuff.fillRect(0, 0, 240, 135,
                   Disbuff.color565(10, 10, 10));
  Disbuff.pushSprite(0, 0);
  delay(500);
}

void MPU6886Test() {
  float accX = 0;
  float accY = 0;
  float accZ = 0;

  double theta = 0, last_theta = 0;
  double phi = 0, last_phi = 0;
  double alpha = 0.2;  //was 0.2

  M5.Imu.getAccelData(&accX, &accY, &accZ);

  accZ = 1 - accZ;  //stabilizes z axis readings when the module id sideways otherwise it jumps at times

  if ((accX < 1) && (accX > -1)) {
    theta = asin(-accX) * 57.295;
    if (accZ != 0) {
      phi = atan(accY / accZ) * 57.295;  //was 57.295
    }

    theta = alpha * theta + (1 - alpha) * last_theta;
    phi = alpha * phi + (1 - alpha) * last_phi;

    Disbuff.fillRect(0, 0, 240, 135, TFT_BLACK);
    Disbuff.setTextSize(2);
    Disbuff.setCursor(10, 0);
    Disbuff.setTextColor(TFT_WHITE);
    //Disbuff.printf("%.2f", theta);
    // Disbuff.printf("%.2f", phi);

    myRA.addValue(phi);  //running averages (roll)
    samples++;
    myRAtheta.addValue(theta);  //running averages (roll)
    samplestheta++;

    Disbuff.setTextSize(2);
    Disbuff.setCursor(150, 0);
    // Disbuff.printf("%.2f", myRAtheta.getAverage());
    Disbuff.setCursor(100, 0);
    // Disbuff.printf("%.2f", myRA.getAverage());
    Disbuff.setTextSize(3);
    Disbuff.setCursor(188, 60);
    phi = myRA.getAverage();
    temp = phi;  //save for later
    theta = myRAtheta.getAverage();

    if (abs(phi * swing) <= 30) {
      Disbuff.printf("%.0f", abs(phi * swing));  //approx 1 degree each
    }

    if (samples == 512)  //buffer
    {
      samples = 0;
      myRA.clear();
    }

    if (samples == 512)  //buffer
    {
      samplestheta = 0;
      myRAtheta.clear();
    }

    Disbuff.fillRect(0, 40, 180, 50, TFT_WHITE);  //bottom
    int x0 = 90;
    int y0 = 65;
    int x1 = 70;
    int y1 = 90;
    int x2 = 110;
    int y2 = 90;

    Disbuff.fillTriangle(90, 64, 70, 40, 110, 40, TFT_BLACK);  //top
    phi = phi + offset;

    while (M5.BtnB.pressedFor(10000) && (offset != 0)) {  //one press
      offset = 0;
      Disbuff.createSprite(240, 135);
      Disbuff.fillRect(0, 20, 240, 135, TFT_BLACK);  //bottom
      Disbuff.setTextSize(3);
      Disbuff.setTextColor(TFT_WHITE);
      Disbuff.setCursor(0, 40);
      Disbuff.printf("   RESET!");
      Disbuff.setCursor(0, 90);
      Disbuff.printf("Exiting...\n");
      Disbuff.pushSprite(0, 0);
      delay(4000);
      M5.update();
    }

    M5.update();
    if (M5.BtnA.pressedFor(2000)) {  //If the key is pressed.
      offset = (0 - temp);           //
      Disbuff.createSprite(240, 135);
      Disbuff.fillRect(0, 20, 240, 135, TFT_BLACK);  //bottom
      Disbuff.setTextSize(3);
      Disbuff.setTextColor(TFT_WHITE);
      Disbuff.setCursor(0, 50);
      Disbuff.printf("Calibrated!");
      Disbuff.setCursor(0, 90);
      Disbuff.printf("Exiting...\n");
      Disbuff.pushSprite(0, 0);
      M5.update();
      delay(4000);
    }

    Disbuff.fillTriangle(x0 + phi * pointer, y0, x1 + phi * pointer, y1, x2 + phi * pointer, y2, TFT_BLACK);  //bottom

    if (((x0 + phi * pointer) < (90 + (span / 2))) && ((x0 + phi * pointer) > (90 - (span / 2)))) {
      Disbuff.fillTriangle(90, 64, 70, 40, 110, 40, TFT_BLACK);                                                 //top
      Disbuff.fillTriangle(x0 + phi * pointer, y0, x1 + phi * pointer, y1, x2 + phi * pointer, y2, TFT_BLACK);  //bottom
     // M5.Beep.mute();

    } else {

      if (((x0 + phi * pointer) > 0) && (x0 + phi * pointer) < 180) {
        Disbuff.fillTriangle(x0 + phi * pointer, y0, x1 + phi * pointer, y1, x2 + phi * pointer, y2, TFT_RED);  //bottom
        Disbuff.fillTriangle(90, 64, 70, 40, 110, 40, TFT_RED);                                                 //top
       // M5.Beep.beep();
      }
    }

    last_theta = theta;
    last_phi = phi;
    Disbuff.pushSprite(0, 0);
    M5.update();
  }
}

void loop() {
  MPU6886Test();
  M5.update();
}
